<?php
/**
 * FormValidator.inc.php
 * Code by Strangecode :: www.strangecode.com :: This document contains copyrighted information
 *
 * The FormValidator class provides a method for validating input from
 * http requests and displaying errors.
 *
 * @requires  codebase/lib/Validator.inc.php
 * @author    Quinn Comendant <quinn@strangecode.com>
 * @version   1.8
 *
 * Example of use:
---------------------------------------------------------------------
// The object that validates form input.
require_once 'codebase/lib/FormValidator.inc.php';
$fv = new FormValidator();

$fv->empty('field_name', sprintf(_("%s cannot be blank."), _("Field name")));
$fv->stringLength('field_name', 0, 255, sprintf(_("%s must be %d-to-%d characters in length."), _("Field name"), 0, 255));
$fv->isInteger('field_name', sprintf(_("%s must be an integer."), _("Field name")));
$fv->checkRegex('field_name', '/^\d{4}$|^$/', true, sprintf(_("%s must be in MMYY format."), _("Field name")));
$fv->numericRange('field_name', 0, 65535, sprintf(_("%s must be a number between %d and %d."), _("Field name"), 0, 65535));
$fv->validatePhone('field_name');
$fv->validateEmail('field_name');
$fv->validateStrDate('field_name', sprintf(_("%s must be a valid date in YYYY-MM-DD format."), _("Field name")));
if (is_null($var)) {
    $fv->addError('field_name', sprintf(_("%s is invalid."), _("Field name")));
}
if ($fv->anyErrors()) {
    // Errors!
}
---------------------------------------------------------------------
 */

// Credit card types are defined in class Validator.

require_once 'codebase/lib/Validator.inc.php';

class FormValidator extends Validator {

    // Array filling with error messages.
    var $errors = array();
    
    // Default error marker.
    var $marker = ' sc-msg-error ';

    /**
     * Return the current list of errors.
     *
     * @return array    an array of errors in the following arrangement:
     *                  keys: the name of the variable with an error
     *                  vals: the message to display for that error
     */
    function getErrorList()
    {
        return $this->errors;
    }

    /**
     * Add an error to the errors stack.
     *
     * @param   string $form_name   The name of the incoming form variable.
     * @param   string $msg         The error message for that form.
     * @param   int    $type        The type of message: MSG_NOTICE,
     *                              MSG_SUCCESS, MSG_WARNING, or MSG_ERR.
     * @param   string $file        __FILE__.
     * @param   string $line        __LINE__.
     */
    function addError($form_name, $msg='', $type=MSG_ERR, $file=null, $line=null)
    {
        $this->errors[] = array(
            'name' => $form_name,
            'message' => $msg,
            'type' => $type,
            'file' => $file,
            'line' => $line
        );
    }

    /**
     * Check whether any errors have been triggered.
     *
     * @param  string $form_name the name of the incoming form variable
     *
     * @return bool   true if any errors were found, or if found for
     *                a variable of $form_name, false otherwise
     */
    function anyErrors($form_name=null)
    {
        if (isset($form_name)) {
            foreach ($this->errors as $err) {
                if ($err['name'] == $form_name) {
                    return true;
                }
            }
            return false;
        } else {
            return (sizeof($this->errors) > 0);            
        }
    }

    /**
     * Reset the error list.
     */
    function resetErrorList()
    {
        $this->errors = array();
    }

    /**
     * Prints the HTML for displaying error messages.
     *
     * @access  public
     * @author  Quinn Comendant <quinn@strangecode.com>
     * @since   15 Jul 2005 01:39:14
     */
    function printErrorMessages()
    {
        $app =& App::getInstance();
        if ($this->anyErrors()) {
            ?><div class="sc-msg"><?php
            $errors = $this->getErrorList();
            foreach ($errors as $e) {
                if ('' != $e['message'] && is_string($e['message'])) {
                    if (error_reporting() > 0 && $app->getParam('display_errors') && isset($e['file']) && isset($e['line'])) {
                        echo "\n<!-- [" . $e['file'] . ' : ' . $e['line'] . '] -->';
                    }
                    switch ($e['type']) {
                    case MSG_ERR:
                        echo '<div class="sc-msg-error">' . $e['message'] . '</div>';
                        break;

                    case MSG_WARNING:
                        echo '<div class="sc-msg-warning">' . $e['message'] . '</div>';
                        break;

                    case MSG_SUCCESS:
                        echo '<div class="sc-msg-success">' . $e['message'] . '</div>';
                        break;

                    case MSG_NOTICE:
                    default:
                        echo '<div class="sc-msg-notice">' . $e['message'] . '</div>';
                        break;
                    }
                }
            }
            ?></div><?php
        }
    }

    /**
     * If this form has an error, print an error marker like "<<".
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $marker    A string to print if there is an error. if
     *                           not provided, use default.
     */
    function err($form_name, $marker=null)
    {
        if ($this->anyErrors($form_name)) {
            if (isset($marker)) {
                echo $marker;
            } else {
                echo $this->marker;
            }
        }
    }

    /**
     * Ensure the length of string is non-zero.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if form is not empty, false otherwise.
     */
    function notEmpty($form_name, $msg='')
    {
        if (parent::notEmpty(getFormData($form_name))) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }
    
    /*
    * We were using the isEmpty method *wrong* all these years and should have been using notEmpty.
    * But the fact is the only use is to ensure a value is not empty, so this function simply becomes 
    * an alias of the one-true notEmpty() function.
    * @since    03 Jun 2006 22:56:46
    */
    function isEmpty($form_name, $msg='')
    {
        $this->notEmpty($form_name, $msg='');
    }

    /**
     * Check whether input is a string.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if form is a string, false otherwise.
     */
    function isString($form_name, $msg='')
    {
        if (parent::isString(getFormData($form_name))) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Check whether input is a number. Allows negative numbers.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if no errors found, false otherwise
     */
    function isNumber($form_name, $msg='')
    {
        if (parent::isNumber(getFormData($form_name))) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * addError if input is NOT an integer. Don't just use is_int() because the
     * data coming from the user is *really* a string.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if value is an integer
     */
    function isInteger($form_name, $msg='', $negative_ok=false)
    {
        if (parent::isInteger(getFormData($form_name), $negative_ok)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Check whether input is a float. Don't just use is_float() because the
     * data coming from the user is *really* a string. Integers will also
     * pass this test.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if value is a float
     */
    function isFloat($form_name, $msg='', $negative_ok=false)
    {
        if (parent::isFloat(getFormData($form_name), $negative_ok)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Check whether input is an array.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if value is a float
     */
    function isArray($form_name, $msg='')
    {
        if (parent::isArray(getFormData($form_name))) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Check whether input matches the specified perl regular expression
     * pattern.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  int    $regex     perl regex that the string must match
     * @param  bool   $valid_on_match       set to false to be valid if match, or true
     *                           to be valid on no match
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if value passes regex test
     */
    function checkRegex($form_name, $regex, $valid_on_match, $msg='')
    {
        if (parent::checkRegex(getFormData($form_name), $regex, $valid_on_match)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Tests if the string length is between specified values. Whitespace excluded for min.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  int    $min       minimum length of string, inclusive
     * @param  int    $max       maximum length of string, inclusive
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if string length is within given boundaries
     */
    function stringLength($form_name, $min, $max, $msg='')
    {
        if (parent::stringLength(getFormData($form_name), $min, $max)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Check whether input is within a valid numeric range.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  int    $min       minimum value of number, inclusive
     * @param  int    $max       maximum value of number, inclusive
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if no errors found, false otherwise
     */
    function numericRange($form_name, $min, $max, $msg='')
    {
        if (parent::numericRange(getFormData($form_name), $min, $max)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

    /**
     * Validates an email address based on the recommendations in RFC 3696.
     * Is more loose than restrictive, to allow the many valid variants of
     * email addresses while catching the most common mistakes.
     * http://www.faqs.org/rfcs/rfc822.html
     * http://www.faqs.org/rfcs/rfc2822.html
     * http://www.faqs.org/rfcs/rfc3696.html
     * http://www.faqs.org/rfcs/rfc1035.html
     *
     * @access  public
     * @param   string  $form_name  The name of the incoming form variable.
     * @return  bool    Validity of address.
     * @author  Quinn Comendant <quinn@strangecode.com>
     */
    function validateEmail($form_name)
    {
		$app =& App::getInstance();

        $email = getFormData($form_name);

        if ('' == trim($email)) {
            // No email address provided, and that's okay
            return true;
        }

        // Validator::validateEmail() returns a value that relates to the VALIDATE_EMAIL_* constants (defined in Validator.inc.php).
        switch (parent::validateEmail($email)) {
        case VALIDATE_EMAIL_REGEX_FAIL:
            // Failed regex match.
            $this->addError($form_name, sprintf(_("<em>%s</em> is not a valid email address."), oTxt($email)));
            $app->logMsg(sprintf('The email address %s is not valid.', getFormData($form_name)), LOG_DEBUG, __FILE__, __LINE__);
            return false;
            break;
        case VALIDATE_EMAIL_LENGTH_FAIL :
            // Failed length requirements.
            $this->addError($form_name, sprintf(_("<em>Email address</em> must contain less than 256 characters."), oTxt($email)));
            $app->logMsg(sprintf('The email address %s must contain less than 256 characters.', getFormData($form_name)), LOG_DEBUG, __FILE__, __LINE__);
            return false;
            break;
        case VALIDATE_EMAIL_MX_FAIL :
            // Failed MX record test.
            $this->addError($form_name, sprintf(_("<em>%s</em> is not a valid email domain name"), oTxt($domain)));
            $app->logMsg(sprintf('The email address %s contains an invalid email domain name (%s).', getFormData($form_name), $domain), LOG_INFO, __FILE__, __LINE__);
            return false;
            break;
        case VALIDATE_EMAIL_SUCCESS :
        default :
            return true;
            break;
        }
    }

    /**
     * Check whether input is a valid phone number. Notice: it is now set
     * to allow characters like - or () or + so people can type in a phone
     * number that looks like: +1 (530) 555-1212
     *
     * @param  string  $form_name the name of the incoming form variable
     *
     * @return bool    true if no errors found, false otherwise
     */
    function validatePhone($form_name)
    {
        $phone = getFormData($form_name);

        return (
            $this->checkRegex($form_name, '/^[0-9 +().-]*$/', true, sprintf(_("The phone number <em>%s</em> is not valid."), $phone))
            && $this->stringLength($form_name, 0, 25, sprintf(_("The phone number <em>%s</em> is too long"), $phone))
        );
    }

    /**
     * Verifies that date can be processed by the strtotime function.
     *
     * @param  string  $form_name the name of the incoming form variable
     * @param  string  $msg       the message to display on error
     *
     * @return bool    true if no errors found, false otherwise
     */
    function validateStrDate($form_name, $msg='')
    {
        $app =& App::getInstance();

        if (parent::validateStrDate(getFormData($form_name))) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            $app->logMsg(sprintf('The string date %s is not valid.', getFormData($form_name)), LOG_DEBUG, __FILE__, __LINE__);
            return false;
        }
    }


    /**
     * Verifies credit card number.
     *
     * @param  string  $form_name   The name of the incoming form variable.
     * @param  string  $cc_type     Optional, card type to do specific checks. One of the CC_TYPE_* constants.
     *
     * @return bool    true if no errors found, false otherwise
     */
    function validateCCNumber($form_name, $cc_type=null)
    {
        $cc_num = getFormData($form_name);
        
        if (parent::validateCCNumber($cc_num, $cc_type)) {
            return true;
        } else {
            $this->addError($form_name, sprintf(_("<em>%s</em> is not a valid credit card number."), $cc_num));
            return false;
        }
    }

    /**
     * Check whether a file was selected for uploading. If file is missing, it's an error.
     *
     * @param  string $form_name the name of the incoming form variable
     * @param  string $msg       the message to display on error
     *
     * @return bool   true if no errors found, false otherwise
     */
    function fileUploaded($form_name, $msg='')
    {
        if (parent::fileUploaded($form_name)) {
            return true;
        } else {
            $this->addError($form_name, $msg);
            return false;
        }
    }

} // THE END

?>